#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Photolab_colorblind_huevision python-fu pour Gimp 2.6
# Copyright Raymond Ostertag 2009
# Licence GPL

# Version 1.0
# - initial release

# Installation : put the photolab_colorblind_huevision.py file in your $HOME/.gimp-2.n/plug-ins.
# On Linux and Mac OSX the file must be executable.
# Documentation : http://www.gimp.org/docs/python/index.html

from gimpfu import *
import os, glob

# i18n
#
import gettext
locale_directory = gimp.locale_directory
gettext.install( "gimp20-photolab" , locale_directory, unicode=True )

#
Photolab_colorblind_huevision_help = _("Add/remove a hue layer on top of layers. For colorblind peoples who can not see full intensity of colors.")                                
Photolab_colorblind_huevision_description = Photolab_colorblind_huevision_help
#
#
Photolab_colorblind_huevisiondecompose_help = _("Add/remove a hue layer on top of layers and decompose in gray, red, green and layers. For colorblind peoples who can not see full intensity of colors.")                                
Photolab_colorblind_huevisiondecompose_description = Photolab_colorblind_huevisiondecompose_help
#

# constant
#
layerOpacity = 75.0

# dialog parameters
#
Standalone_parameters = [
  (PF_IMAGE, "inImage", "Input image", None),
  (PF_DRAWABLE, "inLayer", "Input drawable", None),
]

# Hue level color parameters
#
huelevelRed = 0
huelevelYellow = int( 60.0*255/360 )
huelevelGreen = int(120.0*255/360 )
huelevelCyan = int( 180.0*255/360 )
huelevelBleu = int( 240.0*255/360 )
huelevelMagenta = int( 300.0*255/360 )

# main
#                                         
def python_fu_photolab_colorblind_huelayer( 
  inImage,
  inDrawable,
  huevisionDecompose
  ): 
  gimp.context_push()
  pdb.gimp_image_undo_group_start( inImage )
  #find if huelayer allready exist
  huelayerHueExist = huelayerRedExist = huelayerGreenExist = huelayerBlueExist = huelayerGrayExist = False
  layerList = inImage.layers
  for layerID in layerList:
    if layerID.name == "hue_of_photolab_colorblind_huelayer":
      huelayerHueExist = True
      huelayerHueID = layerID
    if layerID.name == "gray_of_photolab_colorblind_huelayer":
      huelayerGrayExist = True
      huelayerGrayID = layerID
    if layerID.name == "red_of_photolab_colorblind_huelayer":
      huelayerRedExist = True
      huelayerRedID = layerID
    if layerID.name == "green_of_photolab_colorblind_huelayer":
      huelayerGreenExist = True
      huelayerGreenID = layerID
    if layerID.name == "blue_of_photolab_colorblind_huelayer":
      huelayerBlueExist = True
      huelayerBlueID = layerID
  if huelayerHueExist :
    inImage.remove_layer( huelayerHueID )
  if huelayerGrayExist :
    inImage.remove_layer( huelayerGrayID )
  if huelayerRedExist :
    inImage.remove_layer( huelayerRedID )
  if huelayerGreenExist :
    inImage.remove_layer( huelayerGreenID )
  if huelayerBlueExist :
    inImage.remove_layer( huelayerBlueID )
  startfromscratch = not( huelayerHueExist or huelayerGrayExist or huelayerRedExist or huelayerGreenExist or huelayerBlueExist )
  if startfromscratch:
    hsvImage = pdb.plug_in_decompose( inImage, inDrawable, "HSV", True )[0]
    #add saturationlayer with the saturation channel
    saturationLayer = pdb.gimp_layer_new_from_drawable( hsvImage.layers[1], inImage )    
    inImage.add_layer( saturationLayer )
    #add huelayer with the hue channel
    hueLayer = pdb.gimp_layer_new_from_drawable( hsvImage.layers[0], inImage )
    inImage.add_layer( hueLayer )
    #delete hsv image        
    pdb.gimp_image_delete( hsvImage )    
    #
    #make white pixels with saturation 0
    pdb.gimp_threshold( saturationLayer, 0, 0)
    #substract white pixels of saturationlayer from huelayer
    pdb.gimp_by_color_select( saturationLayer, (255,255,255), 0, CHANNEL_OP_REPLACE, False, False, 0, False)
    if not( pdb.gimp_selection_is_empty( inImage )):
      pdb.gimp_edit_clear( hueLayer )
      #make non white pixels transparent in saturationlayer
      pdb.gimp_selection_invert( inImage )    
      pdb.gimp_edit_clear( saturationLayer )    
      pdb.gimp_selection_none( inImage )
    else:
      #make non white pixels transparent in saturationlayer  
      pdb.gimp_edit_clear( saturationLayer )     
    if huevisionDecompose:
      #
      #green layer
      green_hueLayer = hueLayer.copy()
      inImage.add_layer( green_hueLayer )
      mask_green_hueLayer = green_hueLayer.create_mask( ADD_COPY_MASK )
      pdb.gimp_layer_add_mask( green_hueLayer, mask_green_hueLayer )
      pdb.gimp_threshold( mask_green_hueLayer, huelevelYellow, huelevelCyan )
      #
      #blue layer
      blue_hueLayer = hueLayer.copy()
      inImage.add_layer( blue_hueLayer )
      mask_blue_hueLayer = blue_hueLayer.create_mask( ADD_COPY_MASK )
      pdb.gimp_layer_add_mask( blue_hueLayer, mask_blue_hueLayer )
      pdb.gimp_threshold( mask_blue_hueLayer, huelevelCyan, huelevelMagenta )   
      #
      #red layer
      #red_hueLayer = hueLayer.copy()
      #inImage.add_layer( red_hueLayer )
      mask_hueLayer = hueLayer.create_mask( ADD_COPY_MASK )
      pdb.gimp_layer_add_mask( hueLayer, mask_hueLayer )
      pdb.gimp_threshold( mask_hueLayer, huelevelYellow, huelevelMagenta )
      pdb.gimp_invert( mask_hueLayer )
      #
      #color layers
      pdb.gimp_context_set_gradient( "Full saturation spectrum CCW" ) 
      pdb.plug_in_gradmap( inImage, hueLayer )
      pdb.plug_in_gradmap( inImage, green_hueLayer )
      pdb.plug_in_gradmap( inImage, blue_hueLayer )   
      #
      #set name opacity and mode
      saturationLayer.name = "gray_of_photolab_colorblind_huelayer"
      saturationLayer.opacity = float( layerOpacity )
      saturationLayer.mode = NORMAL_MODE
      hueLayer.name = "red_of_photolab_colorblind_huelayer"
      hueLayer.opacity = float( layerOpacity )   
      hueLayer.mode = NORMAL_MODE
      green_hueLayer.name = "green_of_photolab_colorblind_huelayer"
      green_hueLayer.opacity = float( layerOpacity )   
      green_hueLayer.mode = NORMAL_MODE
      blue_hueLayer.name = "blue_of_photolab_colorblind_huelayer"
      blue_hueLayer.opacity = float( layerOpacity )   
      blue_hueLayer.mode = NORMAL_MODE
    else:
      #
      #color layers
      pdb.gimp_context_set_gradient( "Full saturation spectrum CCW" ) 
      pdb.plug_in_gradmap( inImage, hueLayer )  
      #
      #set name opacity and mode
      saturationLayer.name = "gray_of_photolab_colorblind_huelayer"
      saturationLayer.opacity = float( layerOpacity )
      saturationLayer.mode = NORMAL_MODE     
      hueLayer.name = "hue_of_photolab_colorblind_huelayer"
      hueLayer.opacity = float( layerOpacity )
      hueLayer.mode = NORMAL_MODE
    #
  pdb.gimp_image_undo_group_end( inImage )
  gimp.context_pop()
  
def python_fu_photolab_colorblind_huevision( 
  inImage,
  inDrawable,
  ): 
  python_fu_photolab_colorblind_huelayer( inImage, inDrawable, False )
  
def python_fu_photolab_colorblind_huevisiondecompose( 
  inImage,
  inDrawable,
  ): 
  python_fu_photolab_colorblind_huelayer( inImage, inDrawable, True )

register(
  "python-fu-photolab-colorblind-huevision",
  Photolab_colorblind_huevision_description,
  Photolab_colorblind_huevision_help,
  "Raymond Ostertag",
  "GPL License",
  "2009",
  _("Add/Remove hue vision"),
  "*",
  Standalone_parameters,
  [],
  python_fu_photolab_colorblind_huevision,
  menu="<Image>/Colors"+"/"+_("Colorblind tools"),
  domain=("gimp20-photolab", locale_directory)   
  )
  
register(
  "python-fu-photolab-colorblind-huevisiondecompose",
  Photolab_colorblind_huevisiondecompose_description,
  Photolab_colorblind_huevisiondecompose_help,
  "Raymond Ostertag",
  "GPL License",
  "2009",
  _("Add/Remove hue vision RGB decomposed"),
  "*",
  Standalone_parameters,
  [],
  python_fu_photolab_colorblind_huevisiondecompose,
  menu="<Image>/Colors"+"/"+_("Colorblind tools"),
  domain=("gimp20-photolab", locale_directory)   
  )

main()
